# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""All RT-X datasets."""

from tensorflow_datasets.robotics.rtx.rtx import AlohaMobile
from tensorflow_datasets.robotics.rtx.rtx import AsuTableTopConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import AustinBudsDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import AustinSailorDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import AustinSiriusDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import BcZ
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyAutolabUr5
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyCableRouting
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyFanucManipulation
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyGnmCoryHall
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyGnmRecon
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyGnmSacSon
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyMvpConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import BerkeleyRptConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import Bridge
from tensorflow_datasets.robotics.rtx.rtx import BridgeDataMsr
from tensorflow_datasets.robotics.rtx.rtx import CmuFrankaExplorationDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import CmuPlayFusion
from tensorflow_datasets.robotics.rtx.rtx import CmuStretch
from tensorflow_datasets.robotics.rtx.rtx import ColumbiaCairlabPushtReal
from tensorflow_datasets.robotics.rtx.rtx import ConqHoseManipulation
from tensorflow_datasets.robotics.rtx.rtx import DlrEdanSharedControlConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import DlrSaraGridClampConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import DlrSaraPourConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import Dobbe
from tensorflow_datasets.robotics.rtx.rtx import EthAgentAffordances
from tensorflow_datasets.robotics.rtx.rtx import Fmb
from tensorflow_datasets.robotics.rtx.rtx import Fractal20220817Data
from tensorflow_datasets.robotics.rtx.rtx import IamlabCmuPickupInsertConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import ImperialcollegeSawyerWristCam
from tensorflow_datasets.robotics.rtx.rtx import IoAiTech
from tensorflow_datasets.robotics.rtx.rtx import JacoPlay
from tensorflow_datasets.robotics.rtx.rtx import KaistNonprehensileConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import Kuka
from tensorflow_datasets.robotics.rtx.rtx import ManiskillDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import MimicPlay
from tensorflow_datasets.robotics.rtx.rtx import NyuDoorOpeningSurprisingEffectiveness
from tensorflow_datasets.robotics.rtx.rtx import NyuFrankaPlayDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import NyuRotDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import PlexRobosuite
from tensorflow_datasets.robotics.rtx.rtx import RoboSet
from tensorflow_datasets.robotics.rtx.rtx import Roboturk
from tensorflow_datasets.robotics.rtx.rtx import SpocRobot
from tensorflow_datasets.robotics.rtx.rtx import StanfordHydraDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import StanfordKukaMultimodalDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import StanfordMaskVitConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import StanfordRobocookConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import TacoPlay
from tensorflow_datasets.robotics.rtx.rtx import Tidybot
from tensorflow_datasets.robotics.rtx.rtx import TokyoULsmoConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import Toto
from tensorflow_datasets.robotics.rtx.rtx import UcsdKitchenDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UcsdPickAndPlaceDatasetConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UiucD3field
from tensorflow_datasets.robotics.rtx.rtx import UscClothSimConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UtaustinMutex
from tensorflow_datasets.robotics.rtx.rtx import UtokyoPr2OpeningFridgeConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UtokyoPr2TabletopManipulationConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UtokyoSaytapConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UtokyoXarmBimanualConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import UtokyoXarmPickAndPlaceConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import VimaConvertedExternallyToRlds
from tensorflow_datasets.robotics.rtx.rtx import Viola
